#!/usr/bin/env python
#-*- coding: utf-8 -*-
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

import re
import os
import syslog
import json
import sys
import subprocess
from argparse import ArgumentParser

def log (msg):
	syslog.syslog(syslog.LOG_ERR, str(msg))

def callChatHook (type, args = None):
	status = subprocess.call(['/var/packages/Chat/scripts/start-stop-status', 'status'])
	if status != 0:
		return

	pipe = os.popen('/var/packages/Chat/target/tool/synochat_sdkhook ' + type, 'w')
	logMsg = 'chat hook called ' + type

	if args != None:
		argStr = json.dumps(args)
		pipe.write(argStr)
		logMsg += ' -> ' + argStr

	log(logMsg + ', environ: ' + str(os.environ))

def callUpstart (upstartJob, operation):
	subprocess.check_output(['/sbin/initctl', operation, upstartJob])
	log('sdk hook: ' + operation + 'ed ' + upstartJob)

class SynoUser:
	def __init__ (self):
		self.usernameRe = re.compile('\d+:\[(.+)\]')

	def listGroupMembers (self, gid = None, groupName = None):
		method = '--getgid' if groupName == None else '--get'
		value = gid if groupName == None else groupName
		userUIDs = []
		pipe = os.popen('/usr/syno/sbin/synogroup %s %d'.format(method, value), 'r')
		for line in pipe:
			match = self.usernameRe.match(line)
			if match:
				userUIDs.append(int(match.group(1)))

		return userUIDs

class Runner:
	def __init__ (self):
		self.version = '1.0'

	def pre (self):
		pass

	def post (self):
		pass

	def startPackage (self, reason = ''):
		log('Starting package Chat... ' + reason)
		os.system('/usr/syno/bin/synopkg start Chat')

	def stopPackage (self, reason = ''):
		log('Stopping package Chat... ' + reason)
		os.system('/usr/syno/bin/synopkg stop Chat')
		os.system('/usr/syno/bin/synodsmnotify @administrators Chat "package Chat is stopped: ' + reason + '"')

def Main (runner):
	def printPkgInfo (key):
		print os.popen("/bin/get_key_value /var/packages/Chat/INFO " + key, 'r').read() ,

	parser = ArgumentParser(description='Chat SDK Hooks')
	parser.add_argument("--sdk-mod-ver", action = "store_true")
	parser.add_argument("--name", action = "store_true")
	parser.add_argument("--pkg-ver", action = "store_true")
	parser.add_argument("--vendor", action = "store_true")
	parser.add_argument("--pre", action = "store_true")
	parser.add_argument("--post", action = "store_true")
	options = parser.parse_args()

	if options.sdk_mod_ver:
		print runner.version
	elif options.name:
		printPkgInfo("package")
	elif options.pkg_ver:
		printPkgInfo("version")
	elif options.vendor:
		printPkgInfo("maintainer")
	elif options.pre:
		runner.pre()
	elif options.post:
		runner.post()
