UPDATE users SET
	create_at = create_at * 1000,
	update_at = update_at * 1000,
	delete_at = delete_at * 1000;
UPDATE channels SET
	create_at = create_at * 1000,
	update_at = update_at * 1000,
	last_post_at = last_post_at * 1000,
	close_at = close_at * 1000;
UPDATE channel_members SET
	last_hide_at = last_hide_at * 1000,
	last_view_at = last_view_at * 1000,
	create_at = create_at * 1000,
	delete_at = delete_at * 1000;
UPDATE user_stars SET
	create_at = create_at * 1000;
UPDATE configs SET
	update_at = update_at * 1000;
CREATE OR REPLACE FUNCTION get_time_ms(OUT result bigint) AS $$
BEGIN
	result := round(EXTRACT(EPOCH FROM now()) * 1000);
END;
$$ language 'plpgsql';
CREATE OR REPLACE FUNCTION updateat_trigger() RETURNS TRIGGER AS $$
BEGIN
	NEW.update_at = get_time_ms();
	RETURN NEW;
END;
$$ language 'plpgsql';
CREATE OR REPLACE FUNCTION channel_update() RETURNS trigger AS $$
BEGIN
	IF OLD.total_member_count != 0 AND NEW.total_member_count = 0 AND NEW.type = 1 THEN
		NEW.close_at = get_time_ms();
		NEW.name = NULL;
	END IF;
	RETURN NEW;
END;
$$ LANGUAGE plpgsql;
CREATE OR REPLACE FUNCTION post_updateat_trigger() RETURNS TRIGGER AS $$
BEGIN
	IF NEW.update_at IS NULL AND OLD.message != NEW.message THEN
		NEW.update_at = get_time_ms();
	ELSIF OLD.update_at IS NOT NULL AND NEW.update_at < OLD.update_at THEN
		RAISE EXCEPTION 'cannot update older data';
	ELSE
		NEW.update_at = OLD.update_at;
	END IF;
	IF OLD.delete_at IS NULL AND NEW.delete_at IS NOT NULL
		AND OLD.thread_id IS NOT NULL AND OLD.thread_id != OLD.id THEN
		EXECUTE format('UPDATE channel_%s.posts SET comment_count = comment_count - 1 WHERE thread_id = %s AND thread_id = id', OLD.channel_id, OLD.thread_id);
	END IF;
	RETURN NEW;
END;
$$ language 'plpgsql';
CREATE OR REPLACE FUNCTION post_create_trigger() RETURNS trigger AS $$
BEGIN
	UPDATE channels SET last_post_at = get_time_ms() WHERE id = NEW.channel_id;
	IF NEW.thread_id IS NOT NULL THEN
		EXECUTE format('UPDATE channel_%s.posts SET comment_count = comment_count + 1 WHERE thread_id = %s AND thread_id = id', NEW.channel_id, NEW.thread_id);
		EXECUTE format('UPDATE channel_%s.posts SET last_comment_at = %s WHERE thread_id = %s AND thread_id = id',
			NEW.channel_id, NEW.create_at, NEW.thread_id);
	END IF;
	RETURN NEW;
END;
$$ LANGUAGE plpgsql;
ALTER TABLE users ALTER COLUMN create_at SET DEFAULT get_time_ms();
ALTER TABLE channels ALTER COLUMN create_at SET DEFAULT get_time_ms();
ALTER TABLE channel_members ALTER COLUMN create_at SET DEFAULT get_time_ms();
ALTER TABLE posts ALTER COLUMN create_at SET DEFAULT get_time_ms();
ALTER TABLE user_stars ALTER COLUMN create_at SET DEFAULT get_time_ms();
