CREATE TABLE archives (
	id                   serial          NOT NULL PRIMARY KEY,
	channel_id           int             NOT NULL REFERENCES channels(id) ON DELETE CASCADE ON UPDATE CASCADE,
	channel_name         text            NOT NULL,
	user_id              int,
	post_id_start        bigint          NOT NULL,
	post_id_end          bigint          NOT NULL,
	props                json            NOT NULL,
	create_at            bigint          NOT NULL DEFAULT get_time_ms()
);
CREATE UNIQUE INDEX archives_user_id_post_id_start_key ON archives(user_id, post_id_start, channel_id);
CREATE INDEX archives_user_id_channel_name_key ON archives(user_id, channel_name);
CREATE INDEX archives_user_id_key ON archives(user_id);
CREATE VIEW view_archives_channels AS
	SELECT A.*, C.type AS channel_type
	FROM archives AS A
	LEFT JOIN channels AS C ON A.channel_id = C.id;
CREATE OR REPLACE FUNCTION channel_create() RETURNS trigger AS $$
BEGIN
	EXECUTE format('CREATE SCHEMA channel_%s;', NEW.id);
	EXECUTE format('CREATE SEQUENCE channel_%s.posts_id_seq;', NEW.id);
	EXECUTE format('CREATE TABLE channel_%s.posts (LIKE posts INCLUDING ALL);', NEW.id);
	EXECUTE format('ALTER TABLE channel_%s.posts ALTER COLUMN id SET DEFAULT next_post_id(''channel_%s.posts_id_seq'', %s);', NEW.id, NEW.id, NEW.id);
	EXECUTE format('ALTER TABLE channel_%s.posts ADD CONSTRAINT channel_id_chk CHECK ( channel_id = %s );', NEW.id, NEW.id);
	EXECUTE format('ALTER TABLE channel_%s.posts ADD CONSTRAINT thread_id FOREIGN KEY (thread_id) REFERENCES channel_%s.posts(id) ON DELETE CASCADE ON UPDATE CASCADE;', NEW.id, NEW.id);
	EXECUTE format('ALTER TABLE channel_%s.posts INHERIT posts;', NEW.id);
	EXECUTE format('CREATE TRIGGER channel_%s_posts_update_trigger BEFORE UPDATE ON channel_%s.posts FOR EACH ROW EXECUTE PROCEDURE post_updateat_trigger();', NEW.id, NEW.id);
	EXECUTE format('CREATE TRIGGER channel_%s_posts_delete_trigger BEFORE DELETE ON channel_%s.posts FOR EACH ROW EXECUTE PROCEDURE post_delete_trigger();', NEW.id, NEW.id);
	EXECUTE format('CREATE TRIGGER channel_%s_posts_create_trigger AFTER INSERT ON channel_%s.posts FOR EACH ROW EXECUTE PROCEDURE post_create_trigger();', NEW.id, NEW.id);
	EXECUTE format('CREATE VIEW channel_%s.view_current_posts AS SELECT * FROM channel_%s.posts WHERE id >= (SELECT post_id_since FROM channels WHERE id = %s);', NEW.id, NEW.id, NEW.id);
	RETURN NEW;
END;
$$ LANGUAGE plpgsql;
CREATE OR REPLACE FUNCTION channel_delete() RETURNS trigger AS $$
BEGIN
	EXECUTE format('DROP VIEW IF EXISTS channel_%s.view_current_posts;', OLD.id);
	EXECUTE format('DROP TABLE IF EXISTS channel_%s.posts CASCADE;', OLD.id);
	EXECUTE format('DROP SEQUENCE IF EXISTS channel_%s.posts_id_seq;', OLD.id);
	EXECUTE format('DROP SCHEMA IF EXISTS channel_%s CASCADE;', OLD.id);
	RETURN OLD;
END;
$$ LANGUAGE plpgsql;
