CREATE TABLE users_guest (
	id                   serial          NOT NULL PRIMARY KEY,
	user_id              int             REFERENCES users_dsm(id) ON DELETE CASCADE ON UPDATE CASCADE,
	email                text            NOT NULL UNIQUE,
	create_at            bigint          NOT NULL DEFAULT get_time_ms()
);
CREATE INDEX users_guest_user_id_key ON users_guest (user_id);
CREATE TABLE channel_guests (
	id                    serial         NOT NULL PRIMARY KEY,
	guest_id              int            NOT NULL REFERENCES users_guest(id) ON DELETE CASCADE ON UPDATE CASCADE,
	channel_id            int            NOT NULL REFERENCES channels(id) ON DELETE CASCADE ON UPDATE CASCADE,
	expire_at             bigint,
	invitor               int            REFERENCES users_dsm(id) ON DELETE CASCADE ON UPDATE CASCADE,
	props                 json
);
CREATE INDEX channel_guests_guest_id_key ON channel_guests (guest_id);
CREATE INDEX channel_guests_channel_id_key ON channel_guests (channel_id);
CREATE INDEX channel_guests_guest_id_channel_id_key ON channel_guests (guest_id, channel_id);
CREATE INDEX channel_guests_expire_at_key ON channel_guests (expire_at);
CREATE VIEW view_channel_guests AS
	SELECT UG.*, CG.guest_id, CG.channel_id, CG.expire_at, CG.invitor, CG.props
	FROM channel_guests AS CG
	JOIN users_guest AS UG ON CG.guest_id = UG.id;
CREATE TABLE acls (
	id                    int             NOT NULL PRIMARY KEY,
	name                  text            NOT NULL UNIQUE
);
CREATE TABLE acl_roles (
	id                    serial          NOT NULL PRIMARY KEY,
	name                  text            NOT NULL UNIQUE,
	description           text
);
ALTER SEQUENCE acl_roles_id_seq START WITH 101;
CREATE TABLE acl_permissions (
	acl_id                int            NOT NULL REFERENCES acls(id) ON DELETE CASCADE ON UPDATE CASCADE,
	role_id               int            NOT NULL REFERENCES acl_roles(id) ON DELETE CASCADE ON UPDATE CASCADE,
	PRIMARY KEY(acl_id, role_id)
);
CREATE INDEX acl_permissions_acl_id_key ON acl_permissions(acl_id);
CREATE INDEX acl_permissions_role_id_key ON acl_permissions(role_id);
INSERT INTO acls (id, name) VALUES
	(1,   'channel_kick'),
	(2,   'channel_set'),
	(3,   'channel_invite'),
	(4,   'channel_join'),
	(5,   'channel_create'),
	(6,   'channel_close'),
	(7,   'channel_archive'),
	(8,   'channel_anonymous'),
	(9,   'channel_disjoin'),
	(10,  'channel_public'),
	(11,  'channel_encryption'),
	(50,  'webhook_create'),
	(51,  'webhook_read'),
	(100, 'guest_manage');
INSERT INTO acl_roles (id, name) VALUES
	(1, 'system_admin'),
	(2, 'normal_user'),
	(3, 'guest');
INSERT INTO acl_permissions (acl_id, role_id)
	SELECT id AS acl_id, 2 FROM acls;