CREATE TABLE chatbots (
	url           text,
	chatbot_props json
) INHERITS (bots);
CREATE UNIQUE INDEX chatbots_id_key ON chatbots (id);
ALTER TABLE chatbots ADD PRIMARY KEY USING INDEX chatbots_id_key;
CREATE INDEX chatbots_updateat_key ON chatbots (update_at);
CREATE INDEX chatbots_deleteat_key ON chatbots (delete_at);
CREATE INDEX chatbots_sub_type_key ON chatbots (sub_type);
CREATE UNIQUE INDEX chatbots_token_key ON chatbots (token);
CREATE TRIGGER chatbots_update_trigger BEFORE UPDATE ON chatbots FOR EACH ROW EXECUTE PROCEDURE updateat_trigger();
CREATE TRIGGER chatbots_delete_trigger BEFORE DELETE ON chatbots FOR EACH ROW EXECUTE PROCEDURE user_delete_trigger();
CREATE INDEX chatbots_browse_key ON chatbots (create_at,delete_at);
CREATE INDEX chatbots_appid_key on chatbots (app_id);
ALTER TABLE users ALTER COLUMN nickname DROP NOT NULL;
ALTER TABLE users ALTER COLUMN nickname DROP DEFAULT;
UPDATE users SET nickname = NULL WHERE nickname = '';
CREATE UNIQUE INDEX chatbots_nickname_key ON chatbots (nickname);
CREATE TABLE cronjobs (
	id                bigserial        NOT NULL PRIMARY KEY,
	props             json,
	create_at         bigint        NOT NULL DEFAULT get_time_ms(),
	expire_at         bigint        NOT NULL
);
CREATE INDEX cronjobs_expire_at_key on cronjobs (expire_at);
CREATE TABLE user_reminders (
	cronjob_id           bigint          NOT NULL PRIMARY KEY REFERENCES cronjobs(id) ON DELETE CASCADE ON UPDATE CASCADE,
	user_id              int             NOT NULL,
	post_id              bigint          NOT NULL,
	channel_id           int             NOT NULL REFERENCES channels(id) ON DELETE CASCADE ON UPDATE CASCADE,
	create_at            bigint          NOT NULL DEFAULT get_time_ms()
);
CREATE INDEX user_reminders_user_id_post_id_key ON user_reminders (user_id, post_id);
CREATE INDEX user_reminders_user_id_channel_id_key ON user_reminders (user_id, channel_id);
CREATE VIEW view_user_reminders AS
	SELECT UR.*, C.expire_at as remind_at
	FROM user_reminders AS UR
	JOIN cronjobs AS C ON UR.cronjob_id = C.id;
CREATE OR REPLACE FUNCTION post_delete_trigger() RETURNS trigger AS $$
BEGIN
	DELETE FROM user_stars WHERE post_id = OLD.id;
	DELETE FROM user_reminders WHERE post_id = OLD.id;
	RETURN OLD;
END;
$$ LANGUAGE plpgsql;
CREATE TABLE schedule_posts (
	cronjob_id           bigint          NOT NULL PRIMARY KEY REFERENCES cronjobs(id) ON DELETE CASCADE ON UPDATE CASCADE,
	user_id              int             NOT NULL,
	channel_id           int             NOT NULL REFERENCES channels(id) ON DELETE CASCADE ON UPDATE CASCADE,
	message              text            NOT NULL,
	create_at            bigint          NOT NULL DEFAULT get_time_ms()
);
CREATE INDEX schedule_posts_user_id_channel_id_key ON schedule_posts (user_id, channel_id);
CREATE VIEW view_schedule_posts AS
	SELECT SP.*, C.expire_at as send_at
	FROM schedule_posts AS SP
	JOIN cronjobs AS C ON SP.cronjob_id = C.id;
UPDATE bots SET props = '{"is_system":true}' WHERE id = 1;